import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class VravServer implements Runnable
{
	public static final byte PACKET_MESSAGE = 1;
	public static final byte PACKET_LOGIN = 2;
	public static final byte PACKET_LOGOUT = 3;
	
	Map<Integer, Socket> sockets;
	private int socketID = 1;
	private static final long serialVersionUID = -4297335882692216363L;

	Socket socket;
	Thread th;
	private static final int port = 2004;
	
	private volatile boolean th_started;

	public static void main(String[] args)
    {		
        new VravServer().createConnection();
    } 
	
	private void createConnection()
	{
		sockets = new HashMap<>();
		try {
			// otherwise create a listening socket and wait for the other party to connect
			System.out.println("Druha strana este nie je pripravena, cakam na spojenie...");
			ServerSocket srv = new ServerSocket(port);
			while(true){
				socket = srv.accept();
				System.out.println("Vytvoreny socket s ID: " + socketID + " pre prijimanie");
				synchronized(sockets){
					for (Map.Entry<Integer, Socket> entry : sockets.entrySet()){
						notify(socketID, entry.getValue(), PACKET_LOGIN);
					}
					sockets.put(socketID, socket);
				}				
				th_started = false;
				new Thread(this).start();
				while (!th_started){}
				socketID++;
			}		    
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
    public void run()
    {
		int localID = socketID;
		th_started = true;
        while(receiveMessage(localID));
		
		synchronized(sockets){
			sockets.remove(localID);
			for (Map.Entry<Integer, Socket> entry : sockets.entrySet()){
				notify(localID, entry.getValue(), PACKET_LOGOUT);
			}
		}
		System.out.println("Koniec rozhovoru " + localID);
		try {
  		  socket.close();
		} catch (Exception e) { e.printStackTrace(); }
    }
	
	private synchronized void broadcast(int id, String message)
	{
		synchronized(sockets){
			for (Map.Entry<Integer, Socket> entry : sockets.entrySet()){
				if (entry.getKey() != id)
				  sendMessage(entry.getValue(), id, message);
			}
		}
	}

	public void sendMessage(Socket so, int id, String message)
	{
		
		try {
			OutputStream wr = so.getOutputStream();
			byte bts[] = message.getBytes();				
				wr.write(PACKET_MESSAGE);
				wr.write((byte)id);
			    wr.write(bts.length & 255);
			    wr.write(bts.length >> 8);
			    wr.write(bts, 0, bts.length);
			    wr.flush();
				System.out.println("sent msg to " + id + " len: " + bts.length + " msg: " + new String(bts));
				 
			} catch (IOException e) {
				e.printStackTrace();
			}
	}
	
	public void notify(int whoIsChanging, Socket so, byte packet)
	{
		
		    try {
                OutputStream wr = so.getOutputStream();
				wr.write(packet);
				wr.write((byte)whoIsChanging);
			    wr.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
	}
	
	public boolean receiveMessage(int id)
	{
		InputStream rd;
		
		try {
			rd = socket.getInputStream();
			int nbts = rd.read() + (rd.read() << 8);
			byte bts[] = new byte[nbts];
			int i = 0; // how many bytes did we read so far
			do {
				int j = rd.read(bts, i, bts.length - i);
				if (j > 0) i += j;
				else break;
			} while (i < bts.length);
				broadcast(id, new String(bts));
		} catch (IOException e) {
			return false;
		}
		return true;
	}
}
